@@ -4,7 +4,8 @@
 from struct import unpack
 from sys import byteorder
 from typing import (
-    Dict, Optional, Union, List, Tuple, TYPE_CHECKING, cast, Iterable
+    Dict, Optional, Union, List, Tuple, TYPE_CHECKING, cast, Iterable,
+    ByteString
 )
 import warnings
 
@@ -16,11 +17,18 @@ except ImportError:
 
 from pydicom.data import get_palette_files
 from pydicom.uid import UID
+from pydicom.valuerep import VR
 
 if TYPE_CHECKING:  # pragma: no cover
     from pydicom.dataset import Dataset, FileMetaDataset, FileDataset
 
 
+# Lookup table for unpacking bit-packed data
+_UNPACK_LUT: Dict[int, bytes] = {
+    k: bytes(int(s) for s in reversed(f"{k:08b}")) for k in range(256)
+}
+
+
 def apply_color_lut(
     arr: "np.ndarray",
     ds: Optional["Dataset"] = None,
@@ -244,7 +252,7 @@ def apply_modality_lut(arr: "np.ndarray", ds: "Dataset") -> "np.ndarray":
     * DICOM Standard, Part 4, :dcm:`Annex N.2.1.1
       <part04/sect_N.2.html#sect_N.2.1.1>`
     """
-    if 'ModalityLUTSequence' in ds:
+    if ds.get("ModalityLUTSequence"):
         item = cast(List["Dataset"], ds.ModalityLUTSequence)[0]
         nr_entries = cast(List[int], item.LUTDescriptor)[0] or 2**16
         first_map = cast(List[int], item.LUTDescriptor)[1]
@@ -254,7 +262,7 @@ def apply_modality_lut(arr: "np.ndarray", ds: "Dataset") -> "np.ndarray":
 
         # Ambiguous VR, US or OW
         unc_data: Iterable[int]
-        if item['LUTData'].VR == 'OW':
+        if item['LUTData'].VR == VR.OW:
             endianness = '<' if ds.is_little_endian else '>'
             unpack_fmt = '{}{}H'.format(endianness, nr_entries)
             unc_data = unpack(unpack_fmt, cast(bytes, item.LUTData))
@@ -342,7 +350,7 @@ def apply_voi_lut(
       <part04/sect_N.2.html#sect_N.2.1.1>`
     """
     valid_voi = False
-    if 'VOILUTSequence' in ds:
+    if ds.get('VOILUTSequence'):
         ds.VOILUTSequence = cast(List["Dataset"], ds.VOILUTSequence)
         valid_voi = None not in [
             ds.VOILUTSequence[0].get('LUTDescriptor', None),
@@ -408,7 +416,7 @@ def apply_voi(
     * DICOM Standard, Part 4, :dcm:`Annex N.2.1.1
       <part04/sect_N.2.html#sect_N.2.1.1>`
     """
-    if "VOILUTSequence" not in ds:
+    if not ds.get('VOILUTSequence'):
         return arr
 
     if not np.issubdtype(arr.dtype, np.integer):
@@ -436,7 +444,7 @@ def apply_voi(
 
     # Ambiguous VR, US or OW
     unc_data: Iterable[int]
-    if item['LUTData'].VR == 'OW':
+    if item['LUTData'].VR == VR.OW:
         endianness = '<' if ds.is_little_endian else '>'
         unpack_fmt = f'{endianness}{nr_entries}H'
         unc_data = unpack(unpack_fmt, cast(bytes, item.LUTData))
@@ -529,7 +537,7 @@ def apply_windowing(
     ds.BitsStored = cast(int, ds.BitsStored)
     y_min: float
     y_max: float
-    if 'ModalityLUTSequence' in ds:
+    if ds.get('ModalityLUTSequence'):
         # Unsigned - see PS3.3 C.11.1.1.1
         y_min = 0
         item = cast(List["Dataset"], ds.ModalityLUTSequence)[0]
@@ -800,6 +808,50 @@ def dtype_corrected_for_endianness(
     return numpy_dtype
 
 
+def expand_ybr422(src: ByteString, bits_allocated: int) -> bytes:
+    """Return ``YBR_FULL_422`` data expanded to ``YBR_FULL``.
+
+    Uncompressed datasets with a (0028,0004) *Photometric Interpretation* of
+    ``"YBR_FULL_422"`` are subsampled in the horizontal direction by halving
+    the number of Cb and Cr pixels (i.e. there are two Y pixels for every Cb
+    and Cr pixel). This function expands the ``YBR_FULL_422`` data to remove
+    the subsampling and the output is therefore ``YBR_FULL``.
+
+    Parameters
+    ----------
+    src : bytes or bytearray
+        The YBR_FULL_422 pixel data to be expanded.
+    bits_allocated : int
+        The number of bits used to store each pixel, as given by (0028,0100)
+        *Bits Allocated*.
+
+    Returns
+    -------
+    bytes
+        The expanded data (as YBR_FULL).
+    """
+    # YBR_FULL_422 is Y Y Cb Cr (i.e. 2 Y pixels for every Cb and Cr pixel)
+    n_bytes = bits_allocated // 8
+    length = len(src) // 2 * 3
+    dst = bytearray(length)
+
+    step_src = n_bytes * 4
+    step_dst = n_bytes * 6
+    for ii in range(n_bytes):
+        c_b = src[2 * n_bytes + ii::step_src]
+        c_r = src[3 * n_bytes + ii::step_src]
+
+        dst[0 * n_bytes + ii::step_dst] = src[0 * n_bytes + ii::step_src]
+        dst[1 * n_bytes + ii::step_dst] = c_b
+        dst[2 * n_bytes + ii::step_dst] = c_r
+
+        dst[3 * n_bytes + ii::step_dst] = src[1 * n_bytes + ii::step_src]
+        dst[4 * n_bytes + ii::step_dst] = c_b
+        dst[5 * n_bytes + ii::step_dst] = c_r
+
+    return bytes(dst)
+
+
 def _expand_segmented_lut(
     data: Tuple[int, ...],
     fmt: str,
@@ -1099,6 +1151,74 @@ def get_nr_frames(ds: "Dataset") -> int:
     return nr_frames
 
 
+def pack_bits(arr: "np.ndarray", pad: bool = True) -> bytes:
+    """Pack a binary :class:`numpy.ndarray` for use with *Pixel Data*.
+
+    .. versionadded:: 1.2
+
+    Should be used in conjunction with (0028,0100) *Bits Allocated* = 1.
+
+    .. versionchanged:: 2.1
+
+        Added the `pad` keyword parameter and changed to allow `arr` to be
+        2 or 3D.
+
+    Parameters
+    ----------
+    arr : numpy.ndarray
+        The :class:`numpy.ndarray` containing 1-bit data as ints. `arr` must
+        only contain integer values of 0 and 1 and must have an 'uint'  or
+        'int' :class:`numpy.dtype`. For the sake of efficiency it's recommended
+        that the length of `arr` be a multiple of 8 (i.e. that any empty
+        bit-padding to round out the byte has already been added). The input
+        `arr` should either be shaped as (rows, columns) or (frames, rows,
+        columns) or the equivalent 1D array used to ensure that the packed
+        data is in the correct order.
+    pad : bool, optional
+        If ``True`` (default) then add a null byte to the end of the packed
+        data to ensure even length, otherwise no padding will be added.
+
+    Returns
+    -------
+    bytes
+        The bit packed data.
+
+    Raises
+    ------
+    ValueError
+        If `arr` contains anything other than 0 or 1.
+
+    References
+    ----------
+    DICOM Standard, Part 5,
+    :dcm:`Section 8.1.1<part05/chapter_8.html#sect_8.1.1>` and
+    :dcm:`Annex D<part05/chapter_D.html>`
+    """
+    if arr.shape == (0,):
+        return bytes()
+
+    # Test array
+    if not np.array_equal(arr, arr.astype(bool)):
+        raise ValueError(
+            "Only binary arrays (containing ones or zeroes) can be packed."
+        )
+
+    if len(arr.shape) > 1:
+        arr = arr.ravel()
+
+    # The array length must be a multiple of 8, pad the end
+    if arr.shape[0] % 8:
+        arr = np.append(arr, np.zeros(8 - arr.shape[0] % 8))
+
+    arr = np.packbits(arr.astype('u1'), bitorder="little")
+
+    packed: bytes = arr.tobytes()
+    if pad:
+        return packed + b'\x00' if len(packed) % 2 else packed
+
+    return packed
+
+
 def pixel_dtype(ds: "Dataset", as_float: bool = False) -> "np.dtype":
     """Return a :class:`numpy.dtype` for the pixel data in `ds`.
 
@@ -1349,3 +1469,57 @@ def reshape_pixel_array(ds: "Dataset", arr: "np.ndarray") -> "np.ndarray":
                 arr = arr.transpose(1, 2, 0)
 
     return arr
+
+
+def unpack_bits(
+    src: bytes, as_array: bool = True
+) -> Union["np.ndarray", bytes]:
+    """Unpack the bit-packed data in `src`.
+
+    Suitable for use when (0028,0011) *Bits Allocated* or (60xx,0100) *Overlay
+    Bits Allocated* is 1.
+
+    If `NumPy <https://numpy.org/>`_ is available then it will be used to
+    unpack the data, otherwise only the standard library will be used, which
+    is about 20 times slower.
+
+    .. versionchanged:: 2.3
+
+        Added the `as_array` keyword parameter, support for unpacking
+        without NumPy, and added :class:`bytes` as a possible return type
+
+    Parameters
+    ----------
+    src : bytes
+        The bit-packed data.
+    as_array : bool, optional
+        If ``False`` then return the unpacked data as :class:`bytes`, otherwise
+        return a :class:`numpy.ndarray` (default, requires NumPy).
+
+    Returns
+    -------
+    bytes or numpy.ndarray
+        The unpacked data as an :class:`numpy.ndarray` (if NumPy is available
+        and ``as_array == True``) or :class:`bytes` otherwise.
+
+    Raises
+    ------
+    ValueError
+        If `as_array` is ``True`` and NumPy is not available.
+
+    References
+    ----------
+    DICOM Standard, Part 5,
+    :dcm:`Section 8.1.1<part05/chapter_8.html#sect_8.1.1>` and
+    :dcm:`Annex D<part05/chapter_D.html>`
+    """
+    if HAVE_NP:
+        arr = np.frombuffer(src, dtype="u1")
+        arr = np.unpackbits(arr, bitorder="little")
+
+        return arr if as_array else arr.tobytes()
+
+    if as_array:
+        raise ValueError("unpack_bits() requires NumPy if 'as_array = True'")
+
+    return b"".join(map(_UNPACK_LUT.__getitem__, src))
