@@ -295,14 +295,15 @@ class TestDataset:
         assert 'Test' == elem.value
         assert 2 == len(self.ds)
 
-    def test_setdefault_unknown_tag(self, allow_invalid_values):
+    def test_setdefault_unknown_tag(self, dont_raise_on_writing_invalid_value):
         with pytest.warns(UserWarning, match=r"\(8888, 0002\)"):
             elem = self.ds.setdefault(0x88880002, 'foo')
         assert 'foo' == elem.value
         assert 'UN' == elem.VR
         assert 2 == len(self.ds)
 
-    def test_setdefault_unknown_tag_strict(self, enforce_valid_values):
+    def test_setdefault_unknown_tag_strict(
+            self, raise_on_writing_invalid_value):
         with pytest.raises(KeyError, match=r"\(8888, 0004\)"):
             elem = self.ds.setdefault(0x88880004, 'foo')
 
@@ -315,18 +316,20 @@ class TestDataset:
         assert 'Test' == elem.value
         assert 2 == len(self.ds)
 
-    def test_setdefault_unknown_tuple(self, allow_invalid_values):
+    def test_setdefault_unknown_tuple(
+            self, dont_raise_on_writing_invalid_value):
         with pytest.warns(UserWarning, match=r"\(8888, 0002\)"):
             elem = self.ds.setdefault((0x8888, 0x0002), 'foo')
         assert 'foo' == elem.value
         assert 'UN' == elem.VR
         assert 2 == len(self.ds)
 
-    def test_setdefault_unknown_tuple_strict(self, enforce_valid_values):
+    def test_setdefault_unknown_tuple_strict(
+            self, raise_on_writing_invalid_value):
         with pytest.raises(KeyError, match=r"\(8888, 0004\)"):
             elem = self.ds.setdefault((0x8888, 0x0004), 'foo')
 
-    def test_setdefault_use_value(self, allow_invalid_values):
+    def test_setdefault_use_value(self, dont_raise_on_writing_invalid_value):
         elem = self.ds.setdefault((0x0010, 0x0010), "Test")
         assert 'Test' == elem.value
         assert 2 == len(self.ds)
@@ -447,6 +450,12 @@ class TestDataset:
         assert 'John' == self.ds[0x10, 0x10].value
         assert 'Johnny' == self.ds[0x10, 0x12].value
 
+    def test_dir_attr(self):
+        """Dataset.__dir__ returns class attributes"""
+        ds = Dataset()
+        assert hasattr(ds, "is_implicit_VR")
+        assert "is_implicit_VR" in dir(ds)
+
     def test_dir_subclass(self):
         """Dataset.__dir__ returns class specific dir"""
 
@@ -1872,7 +1881,7 @@ class TestFileDataset:
         """Ensure that we don't use inherited dict functionality"""
         ds = Dataset()
         di = dict()
-        expected_diff = {'__class__', '__doc__', '__hash__', 'fromkeys'}
+        expected_diff = {'fromkeys'}
         if sys.version_info[:2] >= (3, 8):
             expected_diff.add('__reversed__')
         if sys.version_info[:2] >= (3, 9):
@@ -1922,6 +1931,15 @@ class TestFileDataset:
         ds.preamble = None
         assert ds == ds2
 
+    def test_deepcopy_without_filename(self):
+        """Regression test for #1571."""
+        file_meta = FileMetaDataset()
+        ds = FileDataset(filename_or_obj="", dataset={}, file_meta=file_meta,
+                         preamble=b"\0" * 128)
+
+        ds2 = copy.deepcopy(ds)
+        assert ds2.filename == ""
+
 
 class TestDatasetOverlayArray:
     """Tests for Dataset.overlay_array()."""
