@@ -1023,6 +1023,8 @@ class TestDatasetCompress:
         assert len(ds.PixelData) == 21370
         assert 'PlanarConfiguration' not in ds
         assert ds['PixelData'].is_undefined_length
+        assert not ds.is_implicit_VR
+        assert ds.is_little_endian
 
     @pytest.mark.skipif(not HAVE_NP, reason="Numpy is unavailable")
     def test_compress_arr(self):
@@ -1030,11 +1032,16 @@ class TestDatasetCompress:
         ds = get_testdata_file("CT_small.dcm", read=True)
         assert hasattr(ds, 'file_meta')
         arr = ds.pixel_array
+        ds.is_implicit_VR = True
+        assert ds.is_little_endian
         del ds.PixelData
         del ds.file_meta
+
         ds.compress(RLELossless, arr, encoding_plugin='pydicom')
         assert ds.file_meta.TransferSyntaxUID == RLELossless
         assert len(ds.PixelData) == 21370
+        assert not ds.is_implicit_VR
+        assert ds.is_little_endian
 
     @pytest.mark.skipif(HAVE_NP, reason="Numpy is available")
     def test_encoder_unavailable(self, monkeypatch):
