@@ -30,10 +30,16 @@ from pydicom.pixel_data_handlers.util import (
     get_j2k_parameters,
     get_nr_frames,
     apply_voi,
-    apply_windowing
+    apply_windowing,
+    pack_bits,
+    unpack_bits,
+    expand_ybr422,
+)
+from pydicom.uid import (
+    ExplicitVRLittleEndian,
+    ImplicitVRLittleEndian,
+    UncompressedPixelTransferSyntaxes,
 )
-from pydicom.uid import (ExplicitVRLittleEndian, ImplicitVRLittleEndian,
-                         UncompressedPixelTransferSyntaxes)
 
 
 # PAL: PALETTE COLOR Photometric Interpretation
@@ -62,6 +68,10 @@ MOD_16_SEQ = get_testdata_file("mlut_18.dcm")
 # WIN: Windowing operation
 WIN_12_1F = get_testdata_file("MR-SIEMENS-DICOM-WithOverlays.dcm")
 VOI_08_1F = get_testdata_file("vlut_04.dcm")
+# 1/1, 1 sample/pixel, 3 frame
+EXPL_1_1_3F = get_testdata_file("liver.dcm")
+# Uncompressed YBR_FULL_422
+EXPL_8_3_1F_YBR422 = get_testdata_file('SC_ybr_full_422_uncompressed.dcm')
 
 
 # Tests with Numpy unavailable
@@ -890,14 +900,18 @@ class TestNumpy_ModalityLUT:
         out = apply_modality_lut(arr, ds)
         assert arr is out
 
+        ds.ModalityLUTSequence = []
+        out = apply_modality_lut(arr, ds)
+        assert arr is out
+
     def test_lutdata_ow(self):
         """Test LUT Data with VR OW."""
         ds = dcmread(MOD_16_SEQ)
         assert ds.is_little_endian is True
         seq = ds.ModalityLUTSequence[0]
         assert [4096, -2048, 16] == seq.LUTDescriptor
-        seq.LUTData = pack('<4096H', *seq.LUTData)
         seq['LUTData'].VR = 'OW'
+        seq.LUTData = pack('<4096H', *seq.LUTData)
         arr = ds.pixel_array
         assert -2048 == arr.min()
         assert 4095 == arr.max()
@@ -941,7 +955,7 @@ class TestNumpy_PaletteColor:
         with pytest.raises(ValueError, match=msg):
             apply_color_lut(ds.pixel_array)
 
-    def test_palette_unknown_raises(self):
+    def test_palette_unknown_raises(self, disable_value_validation):
         """Test using an unknown `palette` raise an exception."""
         ds = dcmread(PAL_08_256_0_16_1F)
         # Palette name
@@ -954,7 +968,7 @@ class TestNumpy_PaletteColor:
         with pytest.raises(ValueError, match=msg):
             apply_color_lut(ds.pixel_array, palette='1.2.840.10008.1.1')
 
-    def test_palette_unavailable_raises(self):
+    def test_palette_unavailable_raises(self, disable_value_validation):
         """Test using a missing `palette` raise an exception."""
         os.rename(self.o_palette, self.n_palette)
         ds = dcmread(PAL_08_256_0_16_1F)
@@ -1141,7 +1155,7 @@ class TestNumpy_PaletteColor:
         assert 32768 == rgba[:, :, 3][0, 0]
         assert (32768 == rgba[:, :, 3]).any()
 
-    def test_well_known_palette(self):
+    def test_well_known_palette(self, disable_value_validation):
         """Test using a well-known palette."""
         ds = dcmread(PAL_08_256_0_16_1F)
         # Drop it to 8-bit
@@ -1839,6 +1853,10 @@ class TestNumpy_ApplyWindowing:
         out = apply_windowing(arr, ds)
         assert [-128, -127, -1, 0, 1, 126, 127] == out.tolist()
 
+        ds.ModalityLUTSequence = []
+        out = apply_windowing(arr, ds)
+        assert [-128, -127, -1, 0, 1, 126, 127] == out.tolist()
+
     def test_rescale_empty(self):
         """Test RescaleSlope and RescaleIntercept being empty."""
         ds = dcmread(WIN_12_1F)
@@ -2051,6 +2069,11 @@ class TestNumpy_ApplyVOI:
         out = apply_voi(arr, ds)
         assert [-128, -127, -1, 0, 1, 126, 127] == out.tolist()
 
+        ds.VOILUTSequence = []
+        out = apply_voi(arr, ds)
+        assert [-128, -127, -1, 0, 1, 126, 127] == out.tolist()
+
+
     def test_voi_lutdata_ow(self):
         """Test LUT Data with VR OW."""
         ds = Dataset()
@@ -2072,6 +2095,7 @@ class TestNumpy_ApplyVOI:
 
 @pytest.mark.skipif(not HAVE_NP, reason="Numpy is not available")
 class TestNumpy_ApplyVOILUT:
+    """Tests for util.apply_voi_lut()"""
     def test_unchanged(self):
         """Test input array is unchanged if no VOI LUT"""
         ds = Dataset()
@@ -2082,6 +2106,10 @@ class TestNumpy_ApplyVOILUT:
         out = apply_voi_lut(arr, ds)
         assert [-128, -127, -1, 0, 1, 126, 127] == out.tolist()
 
+        ds.VOILUTSequence = []
+        out = apply_voi_lut(arr, ds)
+        assert [-128, -127, -1, 0, 1, 126, 127] == out.tolist()
+
     def test_only_windowing(self):
         """Test only windowing operation elements present."""
         ds = Dataset()
@@ -2217,3 +2245,165 @@ class TestGetNrFrames:
         with pytest.warns(None) as w:
             assert ds.NumberOfFrames == get_nr_frames(ds)
             assert not w
+
+
+REFERENCE_PACK_UNPACK = [
+    (b'', []),
+    (b'\x00', [0, 0, 0, 0, 0, 0, 0, 0]),
+    (b'\x01', [1, 0, 0, 0, 0, 0, 0, 0]),
+    (b'\x02', [0, 1, 0, 0, 0, 0, 0, 0]),
+    (b'\x04', [0, 0, 1, 0, 0, 0, 0, 0]),
+    (b'\x08', [0, 0, 0, 1, 0, 0, 0, 0]),
+    (b'\x10', [0, 0, 0, 0, 1, 0, 0, 0]),
+    (b'\x20', [0, 0, 0, 0, 0, 1, 0, 0]),
+    (b'\x40', [0, 0, 0, 0, 0, 0, 1, 0]),
+    (b'\x80', [0, 0, 0, 0, 0, 0, 0, 1]),
+    (b'\xAA', [0, 1, 0, 1, 0, 1, 0, 1]),
+    (b'\xF0', [0, 0, 0, 0, 1, 1, 1, 1]),
+    (b'\x0F', [1, 1, 1, 1, 0, 0, 0, 0]),
+    (b'\xFF', [1, 1, 1, 1, 1, 1, 1, 1]),
+    #              | 1st byte              | 2nd byte
+    (b'\x00\x00', [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]),
+    (b'\x00\x01', [0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0]),
+    (b'\x00\x80', [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]),
+    (b'\x00\xFF', [0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1]),
+    (b'\x01\x80', [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]),
+    (b'\x80\x80', [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1]),
+    (b'\xFF\x80', [1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1]),
+]
+
+
+class TestUnpackBits:
+    """Tests for unpack_bits."""
+
+    @pytest.mark.skipif(not HAVE_NP, reason="Numpy is not available")
+    @pytest.mark.parametrize('src, output', REFERENCE_PACK_UNPACK)
+    def test_unpack_np(self, src, output):
+        """Test unpacking data using numpy."""
+        assert np.array_equal(
+            unpack_bits(src, as_array=True), np.asarray(output)
+        )
+
+        as_bytes = pack(f"{len(output)}B", *output)
+        assert unpack_bits(src, as_array=False) == as_bytes
+
+    @pytest.mark.skipif(HAVE_NP, reason="Numpy is available")
+    @pytest.mark.parametrize('src, output', REFERENCE_PACK_UNPACK)
+    def test_unpack_bytes(self, src, output):
+        """Test unpacking data without numpy."""
+        as_bytes = pack(f"{len(output)}B", *output)
+        assert unpack_bits(src, as_array=False) == as_bytes
+
+        msg = r"unpack_bits\(\) requires NumPy if 'as_array = True'"
+        with pytest.raises(ValueError, match=msg):
+            unpack_bits(src, as_array=True)
+
+
+REFERENCE_PACK_PARTIAL = [
+    #              | 1st byte              | 2nd byte
+    (b'\x00\x40', [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]),  # 15-bits
+    (b'\x00\x20', [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]),
+    (b'\x00\x10', [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]),
+    (b'\x00\x08', [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]),
+    (b'\x00\x04', [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]),
+    (b'\x00\x02', [0, 0, 0, 0, 0, 0, 0, 0, 0, 1]),
+    (b'\x00\x01', [0, 0, 0, 0, 0, 0, 0, 0, 1]),  # 9-bits
+    (b'\x80', [0, 0, 0, 0, 0, 0, 0, 1]),  # 8-bits
+    (b'\x40', [0, 0, 0, 0, 0, 0, 1]),
+    (b'\x20', [0, 0, 0, 0, 0, 1]),
+    (b'\x10', [0, 0, 0, 0, 1]),
+    (b'\x08', [0, 0, 0, 1]),
+    (b'\x04', [0, 0, 1]),
+    (b'\x02', [0, 1]),
+    (b'\x01', [1]),
+    (b'', []),
+]
+
+
+@pytest.mark.skipif(not HAVE_NP, reason="Numpy is not available")
+class TestNumpy_PackBits:
+    """Tests for numpy_handler.pack_bits."""
+
+    @pytest.mark.parametrize('output, input', REFERENCE_PACK_UNPACK)
+    def test_pack(self, input, output):
+        """Test packing data."""
+        assert output == pack_bits(np.asarray(input), pad=False)
+
+    def test_non_binary_input(self):
+        """Test non-binary input raises exception."""
+        with pytest.raises(ValueError,
+                           match=r"Only binary arrays \(containing ones or"):
+            pack_bits(np.asarray([0, 0, 2, 0, 0, 0, 0, 0]))
+
+    def test_ndarray_input(self):
+        """Test non 1D input gets ravelled."""
+        arr = np.asarray(
+            [[0, 0, 0, 0, 0, 0, 0, 0],
+             [1, 0, 1, 0, 1, 0, 1, 0],
+             [1, 1, 1, 1, 1, 1, 1, 1]]
+        )
+        assert (3, 8) == arr.shape
+        b = pack_bits(arr, pad=False)
+        assert b'\x00\x55\xff' == b
+
+    def test_padding(self):
+        """Test odd length packed data is padded."""
+        arr = np.asarray(
+            [[0, 0, 0, 0, 0, 0, 0, 0],
+             [1, 0, 1, 0, 1, 0, 1, 0],
+             [1, 1, 1, 1, 1, 1, 1, 1]]
+        )
+        assert 3 == len(pack_bits(arr, pad=False))
+        b = pack_bits(arr, pad=True)
+        assert 4 == len(b)
+        assert 0 == b[-1]
+
+    @pytest.mark.parametrize('output, input', REFERENCE_PACK_PARTIAL)
+    def test_pack_partial(self, input, output):
+        """Test packing data that isn't a full byte long."""
+        assert output == pack_bits(np.asarray(input), pad=False)
+
+    def test_functional(self):
+        """Test against a real dataset."""
+        ds = dcmread(EXPL_1_1_3F)
+        arr = ds.pixel_array
+        arr = arr.ravel()
+        assert ds.PixelData == pack_bits(arr)
+
+
+@pytest.mark.skipif(not HAVE_NP, reason="Numpy is not available")
+class TestExpandYBR422:
+    def test_8bit(self):
+        """Test 8-bit expansion."""
+        ds = dcmread(EXPL_8_3_1F_YBR422)
+        assert ds.PhotometricInterpretation == 'YBR_FULL_422'
+        ref = ds.pixel_array
+
+        expanded = expand_ybr422(ds.PixelData, ds.BitsAllocated)
+        arr = np.frombuffer(expanded, dtype="u1")
+        assert np.array_equal(arr, ref.ravel())
+
+    def test_16bit(self):
+        """Test 16-bit expansion."""
+        # Have to make our own 16-bit data
+        ds = dcmread(EXPL_8_3_1F_YBR422)
+        ref = ds.pixel_array.astype('float32')
+        ref *= 65535 / 255
+        ref = ref.astype("u2")
+        # Subsample
+        # YY BB RR YY BB RR YY BB RR YY BB RR -> YY YY BB RR YY YY BB RR
+        src = bytearray(ref.tobytes())
+        del src[2::12]
+        del src[2::11]
+        del src[2::10]
+        del src[2::9]
+
+        # Should be 2/3rds of the original number of bytes
+        nr_bytes = ds.Rows * ds.Columns * ds.SamplesPerPixel * 2
+        assert len(src) == nr_bytes * 2 // 3
+        arr = np.frombuffer(expand_ybr422(src, 16), "u2")
+        assert np.array_equal(arr, ref.ravel())
+        # Spot check values
+        arr = arr.reshape(100, 100, 3)
+        assert (19532, 21845, 65535) == tuple(arr[5, 50, :])
+        assert (42662, 27242, 49601) == tuple(arr[15, 50, :])
