@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # Copyright 2008-2019 pydicom authors. See LICENSE file for details.
 import json
+from unittest import mock
 
 import pytest
 
@@ -13,7 +14,7 @@ from pydicom.valuerep import PersonName
 
 
 class TestPersonName:
-    def test_json_pn_from_file(self):
+    def test_json_pn_from_file(self, disable_value_validation):
         with open(get_testdata_file("test_PN.json")) as s:
             ds = Dataset.from_json(s.read())
         assert isinstance(ds[0x00080090].value, PersonName)
@@ -159,7 +160,8 @@ class TestAT:
 
 
 class TestDataSetToJson:
-    def test_json_from_dicom_file(self, no_numpy_use):
+    def test_json_from_dicom_file(self, no_numpy_use,
+                                  disable_value_validation):
         ds1 = dcmread(get_testdata_file("CT_small.dcm"))
         ds_json = ds1.to_json()
         ds2 = Dataset.from_json(ds_json)
@@ -270,13 +272,14 @@ class TestDataSetToJson:
         assert ds_json.index('"00100020"') < ds_json.index('"00100030"')
         assert ds_json.index('"00100030"') < ds_json.index('"00100040"')
 
-    def test_suppress_invalid_tags(self):
+    @mock.patch("pydicom.DataElement.to_json_dict", side_effect=ValueError)
+    def test_suppress_invalid_tags(self, _):
         """Test tags that raise exceptions don't if suppress_invalid_tags True.
         """
         ds = Dataset()
-        ds.add_new(0x00100010, 'PN', ['Jane^Doe'])
+        ds.add_new(0x00100010, 'PN', 'Jane^Doe')
 
-        with pytest.raises(Exception):
+        with pytest.raises(ValueError):
             ds.to_json_dict()
 
         ds_json = ds.to_json_dict(suppress_invalid_tags=True)
@@ -285,7 +288,7 @@ class TestDataSetToJson:
 
 
 class TestSequence:
-    def test_nested_sequences(self):
+    def test_nested_sequences(self, disable_value_validation):
         test1_json = get_testdata_file("test1.json")
         with open(test1_json) as f:
             with pytest.warns(UserWarning,
