@@ -621,9 +621,9 @@ class TM(_DateTimeBase, datetime.time):
     """
     _RE_TIME = re.compile(
         r"(?P<h>^([01][0-9]|2[0-3]))"
-        r"((?P<m>([0-5][0-9]))?"
-        r"(?(5)(?P<s>([0-5][0-9]|60))?)"
-        r"(?(7)(\.(?P<ms>([0-9]{1,6})?))?))$"
+        r"((?P<m>([0-5][0-9]))"
+        r"((?P<s>([0-5][0-9]|60))"
+        r"(\.(?P<ms>([0-9]{1,6})?))?)?)?$"
     )
 
     def __new__(  # type: ignore[misc]
